require "fileutils"

require_relative './test_helper.rb'
require 'jekyll-git-authors.rb'

class TestGit < Minitest::Test
  def setup
    @original_dir = Dir.pwd
    @dir = Dir.mktmpdir 'jekyll-git-authors'

    @subdir = File.join(@dir, 'subdir')

    Dir.mkdir @subdir

    FileUtils.cp './test/fixtures/README.md', @subdir

    Dir.chdir @dir


    Git.execute 'init'

    Git.execute 'config user.email "you@example.com"'
    Git.execute 'config user.name "Your Name"'

    Git.execute 'add .'
    Git.execute 'commit -m "Initial commit"'
  end

  def teardown
    Dir.chdir @original_dir
    FileUtils.rm_rf @dir
  end

  def test_that_git_executes_basic_commands
    assert_match /^git version /, Git.execute('--version')
  end

  def test_that_git_aborts_on_invalid_command
    invalid_command = 'patses'
    assert_output('', /^Git command failed: '#{invalid_command}' /) do
      assert_raises(SystemExit) do
        Git.execute invalid_command
      end
    end
  end

  def test_that_git_aborts_with_custom_message
    invalid_command = 'burger order'
    err_message = "We do not serve burgers"

    assert_output('', /^Git command failed: '#{invalid_command}' #{err_message}/) do
      assert_raises(SystemExit)do
        Git.execute invalid_command, err_message
      end
    end
  end

  def test_git_log
    method_output = Git.log File.join(@subdir, 'README.md')
    assert_equal "Your Name;you@example.com", "#{method_output}"
  end

  def test_latest_author_email_is_used
    output = "Your Name;you@example.com\nYour Name;me@example.com"
    git_log = Git::Log.new output
    assert_equal({"Your Name"=>"you@example.com"}, git_log.author_email)
  end

  def test_that_it_logs_single_author_once_if_author_has_multiple_commits
    output = "Your Name;you@example.com\nYour Name;you@example.com"
    git_log = Git::Log.new output
    assert_equal({"Your Name"=>"you@example.com"}, git_log.author_email)
  end

  def test_that_it_logs_multiple_authors
    output = "Your Name;you@example.com\nMy Name;me@example.org"
    git_log = Git::Log.new output
    assert_equal({"Your Name"=>"you@example.com", "My Name"=>"me@example.org"}, git_log.author_email)
  end

  def test_method_author_email_stops_at_5_authors
    authors = ""
    6.times do |num|
      authors << "Your Name#{num};you#{num}@example.com\n"
    end
    git_log = Git::Log.new authors

    assert_equal 5, git_log.author_email.size
  end

  def test_method_author_email_iterates_through_authors
    authors = ""
    3.times do
      authors << "Your Name0;you0@example.com\n"
    end
    3.times do
      authors << "Your Name1;you1@example.com\n"
    end
    2.upto(8) do |num|
      authors << "Your Name#{num};you#{num}@example.com\n"
    end

    git_log = Git::Log.new authors

    assert_includes git_log.author_email, 'Your Name4'
    assert_equal 5, git_log.author_email.size
  end

  def test_that_git_log_aborts_with_custom_message_when_file_is_not_tracked_in_git
    FileUtils.cp "#{@original_dir}/test/fixtures/README.md", @dir
    file = './README.md'
    err_message = "Git command failed: 'ls-files --error-unmatch ./README.md' File is not added in git: ./README.md"

    assert_output('', /^#{err_message}/) do
      assert_raises(SystemExit) do
        Git.log(file)
      end
    end
  end

  def test_log_called_when_there_is_no_git_repository
    FileUtils.rm_rf "#{@dir}/.git"

    assert_raises(SystemExit) do
      Git.log File.join(@subdir, 'README.md')
    end
  end
end
